@@ -32,8 +32,11 @@
 
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/cpp_api/tiledb"
 
+using namespace tiledb;
+
 static void check_filters(
     const tiledb::FilterList& answer, const tiledb::FilterList& check) {
   REQUIRE(check.nfilters() == answer.nfilters());
@@ -45,7 +48,6 @@ static void check_filters(
 }
 
 TEST_CASE("C++ API: Filter options", "[cppapi][filter]") {
-  using namespace tiledb;
   Context ctx;
 
   // Test filter creation and option setting/getting
@@ -105,7 +107,6 @@ TEST_CASE("C++ API: Filter options", "[cppapi][filter]") {
 }
 
 TEST_CASE("C++ API: Filter lists", "[cppapi][filter]") {
-  using namespace tiledb;
   Context ctx;
 
   Filter f1(ctx, TILEDB_FILTER_BIT_WIDTH_REDUCTION),
@@ -138,14 +139,12 @@ TEST_CASE("C++ API: Filter lists", "[cppapi][filter]") {
   REQUIRE(list.nfilters() == 3);
 }
 
-TEST_CASE("C++ API: Filter lists on array", "[cppapi][filter]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
+TEST_CASE(
+    "C++ API: Filter lists on array",
+    "[cppapi][filter][rest-fails][sc-45554]") {
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
 
   // Create schema with filter lists
   FilterList a1_filters(ctx);
@@ -236,10 +235,6 @@ TEST_CASE("C++ API: Filter lists on array", "[cppapi][filter]") {
   check_filters(a1_filters, schema_r.attribute("a1").filter_list());
   check_filters(a2_filters, schema_r.attribute("a2").filter_list());
   array.close();
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 void write_sparse_array_string_attr(
@@ -247,10 +242,7 @@ void write_sparse_array_string_attr(
     const std::string& array_name,
     std::string& data,
     std::vector<uint64_t>& data_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   // Write to array
   std::vector<int64_t> d1 = {0, 10, 20, 20, 30, 30, 40};
   std::vector<int64_t> d2 = {0, 10, 20, 30, 30, 40, 40};
@@ -263,15 +255,12 @@ void write_sparse_array_string_attr(
   query.set_data_buffer("a1", data).set_offsets_buffer("a1", data_offsets);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    query.submit_and_finalize();
+  } else {
+    query.submit();
+  }
+
   array.close();
 }
 
@@ -280,10 +269,7 @@ void read_and_check_sparse_array_string_attr(
     const std::string& array_name,
     std::string& expected_data,
     std::vector<uint64_t>& expected_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   Array array(ctx, array_name, TILEDB_READ);
   Query query(ctx, array, TILEDB_READ);
 
@@ -296,15 +282,7 @@ void read_and_check_sparse_array_string_attr(
   query.set_offsets_buffer("a1", attr_off);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  query.submit();
 
   // Check the element offsets are properly returned
   CHECK(attr_val == expected_data);
@@ -315,24 +293,13 @@ void read_and_check_sparse_array_string_attr(
 
 TEST_CASE(
     "C++ API: Filter strings with RLE or Dictionary encoding, sparse array",
-    "[cppapi][filter][rle-strings][dict-strings][sparse]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][sparse][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -366,88 +333,36 @@ TEST_CASE(
 
   SECTION("Unordered write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_UNORDERED,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
   SECTION("Global order write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 void write_dense_array_string_attr(
@@ -455,10 +370,7 @@ void write_dense_array_string_attr(
     const std::string& array_name,
     std::string& data,
     std::vector<uint64_t>& data_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   Array array(ctx, array_name, TILEDB_WRITE);
   Query query(ctx, array, TILEDB_WRITE);
 
@@ -467,15 +379,11 @@ void write_dense_array_string_attr(
   query.set_layout(layout);
   query.set_subarray(Subarray(ctx, array).set_subarray<int64_t>({0, 1, 0, 2}));
 
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    query.submit_and_finalize();
+  } else {
+    query.submit();
+  }
 
   array.close();
 }
@@ -484,10 +392,7 @@ void read_and_check_dense_array_string_attr(
     tiledb::Context ctx,
     const std::string& array_name,
     std::string& expected_data,
-    std::vector<uint64_t>& expected_offsets,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    std::vector<uint64_t>& expected_offsets) {
   Array array(ctx, array_name, TILEDB_READ);
   Query query(ctx, array, TILEDB_READ);
 
@@ -500,15 +405,7 @@ void read_and_check_dense_array_string_attr(
   query.set_offsets_buffer("a1", attr_off);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  query.submit();
 
   // Check the element offsets are properly returned
   CHECK(attr_val == expected_data);
@@ -519,27 +416,13 @@ void read_and_check_dense_array_string_attr(
 
 TEST_CASE(
     "C++ API: Filter strings with RLE or Dictionary encoding, dense array",
-    "[cppapi][filter][rle-strings][dict-strings][dense]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  // Create the array
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][dense][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
+
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -573,55 +456,28 @@ TEST_CASE(
 
   SECTION("Ordered write") {
     write_dense_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_ROW_MAJOR,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     read_and_check_dense_array_string_attr(
-        ctx, array_name, a1_data, a1_offsets, serialized, refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets);
   }
   SECTION("Global order write") {
     write_dense_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     read_and_check_dense_array_string_attr(
-        ctx, array_name, a1_data, a1_offsets, serialized, refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets);
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 TEST_CASE(
     "C++ API: Filter UTF-8 strings with RLE or Dictionary encoding, sparse "
     "array",
-    "[cppapi][filter][rle-strings][dict-strings][sparse][utf-8]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][sparse][utf-8][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -659,104 +515,48 @@ TEST_CASE(
 
   SECTION("Unordered write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_UNORDERED,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
   SECTION("Global order write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 TEST_CASE(
     "C++ API: Filter buffer with some empty strings with RLE or Dictionary "
     "encoding",
-    "[cppapi][filter][rle-strings][dict-strings][empty-strings]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][empty-strings][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create array with string dimension and one attribute
   ArraySchema schema(ctx, TILEDB_SPARSE);
 
@@ -838,8 +638,4 @@ TEST_CASE(
   }
 
   array_r.close();
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
