@@ -33,8 +33,8 @@
 #ifndef TILEDB_TEST_HELPERS_H
 #define TILEDB_TEST_HELPERS_H
 
-#include <tiledb/common/logger_public.h>
 #include "test/support/src/coords_workaround.h"
+#include "test/support/src/mem_helpers.h"
 #include "tiledb.h"
 #include "tiledb/common/common.h"
 #include "tiledb/common/random/random_label.h"
@@ -51,6 +51,15 @@
 #include <string>
 #include <thread>
 
+/**
+ * Helper function to set environment variables across platforms.
+ *
+ * @param __name Name of the environment variable.
+ * @param __value Value of the environment variable.
+ * @return 0 on success, -1 on error.
+ */
+int setenv_local(const char* __name, const char* __value);
+
 // A mutex for protecting the thread-unsafe Catch2 macros.
 extern std::mutex catch2_macro_mutex;
 
@@ -241,10 +250,6 @@ template <class T>
 void check_subarray(
     tiledb::Subarray& subarray, const SubarrayRanges<T>& ranges);
 
-template <class T>
-void check_subarray_equiv(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-
 template <class T>
 bool subarray_equiv(
     tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
@@ -494,7 +499,8 @@ void get_supported_fs(
     bool* s3_supported,
     bool* hdfs_supported,
     bool* azure_supported,
-    bool* gcs_supported);
+    bool* gcs_supported,
+    bool* rest_s3_supported);
 
 /**
  * Opens an array.
@@ -755,12 +761,24 @@ void read_array(
     tiledb_layout_t layout,
     const QueryBuffers& buffers);
 
+/**
+ * Returns the number of commits in the input array,
+ * given a context.
+ */
+int32_t num_commits(Context ctx, const std::string& array_name);
+
 /**
  * Returns the number of commits in the input array,
  * appropriately excluding special files and subdirectories.
  */
 int32_t num_commits(const std::string& array_name);
 
+/**
+ * Returns the number of fragments in the input array,
+ * given a context.
+ */
+int32_t num_fragments(Context ctx, const std::string& array_name);
+
 /**
  * Returns the number of fragments in the input array,
  * appropriately excluding special files and subdirectories.
@@ -899,99 +917,6 @@ int deserialize_array_and_query(
     const char* array_uri,
     bool clientside);
 
-/**
- * Helper method that wraps tiledb_array_open() and inserts a serialization
- * step, if serialization is enabled. This wrapper models exclusively the
- * refactored array open (array open v2) serialization path. The added
- * serialization steps are designed to closely mimic the behavior of the REST
- * server.
- *
- * @param ctx Context.
- * @param query_type Type of query to open the array for.
- * @param serialize True if this is a remote array open, false if not.
- * @param open_array Output open array.
- */
-int array_open_wrapper(
-    tiledb_ctx_t* ctx,
-    tiledb_query_type_t query_type,
-    bool serialize,
-    tiledb_array_t** open_array);
-
-/**
- * C API
- * Helper method that wraps tiledb_query_submit() and inserts serialization
- * steps, if serialization is enabled. The added serialization steps are
- * designed to closely mimic the behavior of the REST server.
- *
- * @param ctx Context.
- * @param array_uri URI of the array the query submit is for.
- * @param query Input/Output tiledb query object to submit/get back.
- * @param buffers Stack allocated buffers to be used by the step simulating
- * "server side buffer allocation" for READ queries. They can only be allocated
- * e.g. once in the test Fixture definition or just before the call to
- * submit_query_wrapper.
- * @param serialize_query True if this is a remote array open, false if not.
- * @param refactored_query_v2 If
- * "rest.use_refactored_array_open_and_query_submit" should be true.
- * @param finalize Finalize or not the query after submitting it.
- */
-int submit_query_wrapper(
-    tiledb_ctx_t* ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2 = false,
-    bool finalize = true);
-
-/** C++ wrapper of submit_query_wrapper */
-int submit_query_wrapper(
-    const Context& ctx,
-    const std::string& array_uri,
-    Query* query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2 = false,
-    bool finalize = true);
-
-/**
- * C API
- * Helper method that wraps tiledb_query_finalize() and inserts serialization
- * steps, if serialization is enabled. The added serialization steps are
- * designed to closely mimic the behavior of the REST server.
- *
- * @param ctx Context.
- * @param array_uri URI of the array the query finalize is for.
- * @param query Input/Output tiledb query object to submit/get back.
- * @param serialize_query True if this is a remote array open, false if not.
- */
-int finalize_query_wrapper(
-    const Context& ctx,
-    const std::string& array_uri,
-    Query* query,
-    bool serialize_query);
-
-/** C++ wrapper of finalize_query_wrapper */
-int finalize_query_wrapper(
-    tiledb_ctx_t* ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    bool serialize_query);
-
-/**
- * Helper function that allocates buffers on a query object that has been
- * deserialized on the "server" side. This is a necassary step for serialized
- * READ queries.
- *
- * @param ctx Context.
- * @param query Query for which we are allocating the buffers.
- * @param query_buffers Input/Output allocated buffers.
- */
-void allocate_query_buffers_server_side(
-    tiledb_ctx_t* ctx,
-    tiledb_query_t* query,
-    ServerQueryBuffers& query_buffers);
-
 }  // namespace tiledb::test
 
 #endif
