@@ -35,6 +35,7 @@
 
 #include "tiledb/common/common.h"
 #include "tiledb/common/macros.h"
+#include "tiledb/common/pmr.h"
 #include "tiledb/common/status.h"
 #include "tiledb/common/types/dynamic_typed_datum.h"
 #include "tiledb/common/types/untyped_datum.h"
@@ -58,6 +59,7 @@ class ConstBuffer;
 class Dimension;
 class DomainTypedDataView;
 class FilterPipeline;
+class MemoryTracker;
 enum class Datatype : uint8_t;
 enum class Layout : uint8_t;
 
@@ -74,20 +76,18 @@ class Domain {
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
 
-  /** Empty constructor. */
-  Domain();
+  /** Deleted default constructor */
+  Domain() = delete;
+
+  /** Constructor. */
+  Domain(shared_ptr<MemoryTracker> memory_tracker);
 
   /** Constructor.*/
   Domain(
       Layout cell_order,
       const std::vector<shared_ptr<Dimension>> dimensions,
-      Layout tile_order);
-
-  /** Copy constructor. */
-  Domain(const Domain&) = default;
-
-  /** Move constructor. */
-  Domain(Domain&& rhs);
+      Layout tile_order,
+      shared_ptr<MemoryTracker> memory_tracker);
 
   /** Destructor. */
   ~Domain() = default;
@@ -96,11 +96,8 @@ class Domain {
   /*             OPERATORS             */
   /* ********************************* */
 
-  /** Copy-assignment operator. */
-  DISABLE_COPY_ASSIGN(Domain);
-
-  /** Move-assignment operator. */
-  Domain& operator=(Domain&& rhs);
+  DISABLE_COPY_AND_COPY_ASSIGN(Domain);
+  DISABLE_MOVE_AND_MOVE_ASSIGN(Domain);
 
   /* ********************************* */
   /*                 API               */
@@ -186,6 +183,7 @@ class Domain {
    * @param cell_order Cell order.
    * @param tile_order Tile order.
    * @param coords_filters Coords filters to replace empty coords pipelines.
+   * @param memory_tracker The memory tracker to use.
    * @return Status and Domain
    */
   static shared_ptr<Domain> deserialize(
@@ -193,7 +191,8 @@ class Domain {
       uint32_t version,
       Layout cell_order,
       Layout tile_order,
-      FilterPipeline& coords_filters);
+      FilterPipeline& coords_filters,
+      shared_ptr<MemoryTracker> memory_tracker);
 
   /** Returns the cell order. */
   Layout cell_order() const;
@@ -295,23 +294,6 @@ class Domain {
   template <class T>
   void get_tile_coords(const T* coords, T* tile_coords) const;
 
-  /**
-   * Retrieves the end of a cell slab starting at the `start` input
-   * coordinates. The cell slab is determined based on the domain
-   * tile/cell order and the input query `layout`. Essentially a
-   * cell slab is a contiguous (in the global cell order) range of
-   * cells that follow the query layout.
-   *
-   * @tparam T The domain type.
-   * @param subarray The subarray in which the end of the cell slab must
-   *     be contained.
-   * @param start The start coordinates.
-   * @param layout The query layout.
-   * @param end The cell slab end coordinates to be retrieved.
-   */
-  template <class T>
-  void get_end_of_cell_slab(T* subarray, T* start, Layout layout, T* end) const;
-
   /**
    * Retrieves the next tile coordinates along the array tile order within a
    * given tile domain. Applicable only to **dense** arrays.
@@ -404,10 +386,9 @@ class Domain {
    * Gets the index in the domain of a given dimension name
    *
    * @param name Name of dimension to check for
-   * @param dim_idx The index of this dimension in the domain
-   * @return Status
+   * @return Dimension index
    */
-  Status get_dimension_index(const std::string& name, unsigned* dim_idx) const;
+  unsigned get_dimension_index(const std::string& name) const;
 
   /** Returns true if at least one dimension has null tile extent. */
   bool null_tile_extents() const;
@@ -505,6 +486,9 @@ class Domain {
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** The memory tracker for this Domain. */
+  shared_ptr<MemoryTracker> memory_tracker_;
+
   /** The number of cells per tile. Meaningful only for the **dense** case. */
   uint64_t cell_num_per_tile_;
 
@@ -520,7 +504,7 @@ class Domain {
    *
    * @invariant All pointers in the vector are non-null.
    */
-  std::vector<shared_ptr<Dimension>> dimensions_;
+  tdb::pmr::vector<shared_ptr<Dimension>> dimensions_;
 
   /**
    * Non-allocating mirror of the dimensions vector.
@@ -532,7 +516,7 @@ class Domain {
    *
    * @invariant All pointers in the vector are non-null.
    */
-  std::vector<const Dimension*> dimension_ptrs_;
+  tdb::pmr::vector<const Dimension*> dimension_ptrs_;
 
   /** The number of dimensions. */
   unsigned dim_num_;
@@ -548,7 +532,7 @@ class Domain {
    * - buff: The buffer that stores all coorinates;
    * - a, b: The positions of the two coordinates in the buffer to compare.
    */
-  std::vector<int (*)(
+  tdb::pmr::vector<int (*)(
       const Dimension* dim, const UntypedDatumView a, const UntypedDatumView b)>
       cell_order_cmp_func_;
 
@@ -558,7 +542,7 @@ class Domain {
    *
    * - coord_a, coord_b: The two coordinates to compare.
    */
-  std::vector<int (*)(const void* coord_a, const void* coord_b)>
+  tdb::pmr::vector<int (*)(const void* coord_a, const void* coord_b)>
       cell_order_cmp_func_2_;
 
   /**
@@ -568,7 +552,7 @@ class Domain {
    * - dim: The dimension to compare on.
    * - coord_a, coord_b: The two coordinates to compare.
    */
-  std::vector<int (*)(
+  tdb::pmr::vector<int (*)(
       const Dimension* dim, const void* coord_a, const void* coord_b)>
       tile_order_cmp_func_;
 
